package org.garret.metronome;

import javax.microedition.lcdui.*;
import java.util.*;
import com.nokia.mid.sound.*;
import com.nokia.mid.ui.DeviceControl;

public class MetronomeCanvas extends Canvas implements CommandListener, SoundListener { 
    Metronome metronome;
    Timer     timer;
    boolean   seal;
    boolean   on;
    long      prevTime;
    int       screenWidth;
    int       screenHeight;
    TimerTask task;
    Font      font;

    static final int WHITE = 0xFFFFFF;
    static final int BLACK = 0;
    static final int FREQ_STEP = 10;
    static final int INPUT_DELAY = 2000;
    static final int MAX_VIBRATION_FREQUENCY = 100;

    public void soundStateChanged(Sound sound, int event) { 
        //System.out.println("soundStateChanged event=" + event);
        if (event == Sound.SOUND_STOPPED) {
            if (!seal) { 
                on = false;
            }
            repaint();
        }
    }


    MetronomeCanvas(Metronome metronome) { 
        this.metronome = metronome;
        timer = new Timer();
	screenWidth = getWidth();
	screenHeight = getHeight();
        font = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_BOLD, Font.SIZE_LARGE);
        setCommandListener(this);
        addCommand(Metronome.QUIT_CMD);
        addCommand(Metronome.SETTINGS_CMD);
        addCommand(Metronome.HELP_CMD); 
        Display.getDisplay(metronome).setCurrent(this);
        metronome.sound.setSoundListener(this);
        restart();
    }

    class MetronomeTask extends TimerTask { 
        public void run() { 
            //System.out.println("Metronome fired");
            on = true;
            metronome.sound.play(1);                        
            //System.out.println("metronome.sound.played");
            if ((metronome.flags & Metronome.VIBRA) != 0) { 
                try { 
                    DeviceControl.startVibra(MAX_VIBRATION_FREQUENCY, metronome.duration);
                } catch (Exception x) { 
                    metronome.flags &= ~Metronome.VIBRA;
                }
            }                    
            repaint();
        }
    }

    void restart() {
        //System.out.println("Restart rate=" + metronome.rate);
        if (task != null) { 
            task.cancel();
            task = null;
        }
        metronome.saveSettings();
        if (metronome.rate != 0) { 
            //System.out.println("Schedule timer with interval=" + (60*1000/metronome.rate));
            task = new MetronomeTask();
            timer.scheduleAtFixedRate(task, 0, 60*1000/metronome.rate);
        } else { 
            repaint();
        }
    }


    protected void paint(Graphics g) {
        //System.out.println("point on=" + on + ", seal=" + seal);
        if (on) {
            g.setColor(WHITE); 
            if ((metronome.flags & Metronome.BACKLIGHT) != 0) {
                //System.out.println("Set lights on");
                DeviceControl.setLights(0, 100);
            }
        } else { 
            g.setColor(BLACK); 
            if ((metronome.flags & Metronome.BACKLIGHT) != 0) {
                //System.out.println("Set lights off");
                DeviceControl.setLights(0, 0);
            }
        }
        g.fillRect(0, 0, screenWidth, screenHeight);        
        if (!seal) { 
            g.setFont(font);        
            g.setColor(on ? BLACK : WHITE); 
            String rate = Integer.toString(metronome.rate);
            g.drawString(rate, screenWidth/2, screenHeight/2, Graphics.HCENTER|Graphics.BASELINE);
            on = false;
        }
        //System.out.println("End of paint");
    }
            
    public void keyReleased(int keyCode)
    {
        if (keyCode == KEY_POUND) { 
            on = seal = false;
            repaint();
        }
    }


    public void keyPressed(int keyCode)
    {
        switch (keyCode) 
        {
        case KEY_NUM0:
        case KEY_NUM1:
        case KEY_NUM2:
        case KEY_NUM3:
        case KEY_NUM4:
        case KEY_NUM5:
        case KEY_NUM6:
        case KEY_NUM7:
        case KEY_NUM8:
        case KEY_NUM9:
        {
            seal = false;
            long currTime = System.currentTimeMillis();
            if (currTime - prevTime > INPUT_DELAY) { 
                metronome.rate = keyCode - KEY_NUM0;
            } else { 
                metronome.rate = metronome.rate*10 + keyCode - KEY_NUM0;
            }
            prevTime = currTime;            
            restart();
            break;            
        }
        case KEY_STAR:
            if (on) { 
                on = seal = false;
            } else { 
                on = seal = true;
            }
            repaint();
            break;
        case KEY_POUND:
            on = seal = true;
            repaint();
            break;
        default:
            switch (getGameAction(keyCode)) { 
            case LEFT:
                if (metronome.rate > 0) { 
                    metronome.rate -= 1;
                }
                break;
            case RIGHT:
                metronome.rate += 1;
                break;
            case UP:
                metronome.rate += FREQ_STEP;
                break;
            case DOWN:
                if (metronome.rate >= FREQ_STEP) { 
                    metronome.rate -= FREQ_STEP;
                }
                break;
            default:
                return;
            }
            restart();
        }
    }


    public void commandAction(Command c, Displayable d) {
        if (c == Metronome.QUIT_CMD) { 
            metronome.destroyApp(true);
            metronome.notifyDestroyed();
        } else if (c == Metronome.SETTINGS_CMD) { 
            new SettingsForm(metronome);
        } else if (c == Metronome.HELP_CMD) { 
            new HelpForm(metronome);
        }
    }         
}